/**********************************************************************************
 * Source file for the implementation of the algorithm for
 * computing rearrangement distance of all permutations of size n.
 * Author : Gustavo Rodrigues Galvao - May 2010.
**********************************************************************************/


#include "loaders.h"

void *loadVectorR(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int i,j;

	*comb = 0;
	for (i = 1; i <= n-1; i++) {
		for (j = i+1; j <= n; j++) {
			vector[*comb] = nextReversal(permutation,temp,copy,i,j);
			(*comb)++;
		}
	}
}

void *loadVectorPR(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int j;

	*comb = 0;
	for (j = 2; j <= n; j++) {
		vector[*comb] = nextReversal(permutation,temp,copy,1,j);
		(*comb)++;
	}
}


void *loadVectorT(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int i,j,k;

	*comb = 0;
	for (i = 1; i <= n-1; i++) {
		for (j = i+1; j <= n; j++) {
			for(k = j+1; k <= n+1; k++) {
				vector[*comb] = nextTransposition(permutation,temp,copy,i,j,k);
				(*comb)++;
			}
		}
	}
}

void *loadVectorPT(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int j,k;

	*comb = 0;
	for (j = 2; j <= n; j++) {
		for(k = j+1; k <= n+1; k++) {
			vector[*comb] = nextTransposition(permutation,temp,copy,1,j,k);
			(*comb)++;
		}
	}
}

void *loadVectorRT(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int i,j,k;

	*comb = 0;
	for (i = 1; i <= n-1; i++) {
		for (j = i+1; j <= n; j++) {
			vector[*comb] = nextReversal(permutation,temp,copy,i,j);
			(*comb)++;
		}
	}

	for (i = 1; i <= n-1; i++) {
		for (j = i+1; j <= n; j++) {
			for(k = j+1; k <= n+1; k++) {
				vector[*comb] = nextTransposition(permutation,temp,copy,i,j,k);
				(*comb)++;
			}
		}
	}
}

void *loadVectorPRPT(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int j,k;

	*comb = 0;
	for (j = 2; j <= n; j++) {
		vector[*comb] = nextReversal(permutation,temp,copy,1,j);
		(*comb)++;
	}

	for (j = 2; j <= n; j++) {
		for(k = j+1; k <= n+1; k++) {
			vector[*comb] = nextTransposition(permutation,temp,copy,1,j,k);
			(*comb)++;
		}
	}
}

void *loadVectorSR(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int i,j;

	*comb = 0;
	for (i = 1; i <= n; i++) {
		for (j = i; j <= n; j++) {
			vector[*comb] = nextSignedReversal(permutation,temp,copy,i,j);
			(*comb)++;
		}
	}
}

void *loadVectorPSR(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int j;

	*comb = 0;
	for (j = 1; j <= n; j++) {
		vector[*comb] = nextSignedReversal(permutation,temp,copy,1,j);
		(*comb)++;
	}
}

void *loadVectorSRT(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int i,j,k;

	*comb = 0;
	for (i = 1; i <= n; i++) {
		for (j = i; j <= n; j++) {
			vector[*comb] = nextSignedReversal(permutation,temp,copy,i,j);
			(*comb)++;
		}
	}

	for (i = 1; i <= n-1; i++) {
		for (j = i+1; j <= n; j++) {
			for(k = j+1; k <= n+1; k++) {
				vector[*comb] = nextTransposition(permutation,temp,copy,i,j,k);
				(*comb)++;
			}
		}
	}
}

void *loadVectorBI(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int i,j,k,l;

	*comb = 0;
	for (i = 1; i <= n-1; i++) {
		for (j = i+1; j <= n; j++) {
			for(k = j; k <= n; k++) {
				for(l = k+1; l <= n+1; l++){
					vector[*comb] = nextBlockInterchange(permutation,temp,copy,i,j,k,l);
					(*comb)++;
				}
			}
		}
	}
}

void *loadVectorPBI(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int j,k,l;

	*comb = 0;
	for (j = 2; j <= n; j++) {
		for(k = j; k <= n; k++) {
			for(l = k+1; l <= n+1; l++){
				vector[*comb] = nextBlockInterchange(permutation,temp,copy,1,j,k,l);
				(*comb)++;
			}
		}
	}
}

void *loadVectorRTTr(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int i,j,k;

	*comb = 0;
	for (i = 1; i <= n-1; i++) {
		for (j = i+1; j <= n; j++) {
			for(k = j+1; k <= n+1; k++) {
				vector[*comb] = nextTransposition(permutation,temp,copy,i,j,k);
				(*comb)++;
				vector[*comb] = nextTransreversalA(permutation,temp,copy,i,j,k);
				(*comb)++;
				vector[*comb] = nextTransreversalB(permutation,temp,copy,i,j,k);
				(*comb)++;
			}
		}
	}
}

void *loadVectorSRTTrAB(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int i,j,k;

	*comb = 0;
	for (i = 1; i <= n-1; i++) {
		for (j = i+1; j <= n; j++) {
			for(k = j+1; k <= n+1; k++) {
				vector[*comb] = nextTransposition(permutation,temp,copy,i,j,k);
				(*comb)++;
				vector[*comb] = nextSignedTransreversalA(permutation,temp,copy,i,j,k);
				(*comb)++;
				vector[*comb] = nextSignedTransreversalB(permutation,temp,copy,i,j,k);
				(*comb)++;
			}
		}
	}

	for (i = 1; i <= n; i++) {
		for (j = i; j <= n; j++) {
			vector[*comb] = nextSignedReversal(permutation,temp,copy,i,j);
			(*comb)++;
		}
	}
}

void *loadVectorSRTTrA(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int i,j,k;

	*comb = 0;
	for (i = 1; i <= n-1; i++) {
		for (j = i+1; j <= n; j++) {
			for(k = j+1; k <= n+1; k++) {
				vector[*comb] = nextTransposition(permutation,temp,copy,i,j,k);
				(*comb)++;
				vector[*comb] = nextSignedTransreversalA(permutation,temp,copy,i,j,k);
				(*comb)++;
			}
		}
	}

	for (i = 1; i <= n; i++) {
		for (j = i; j <= n; j++) {
			vector[*comb] = nextSignedReversal(permutation,temp,copy,i,j);
			(*comb)++;
		}
	}
}

void *loadVectorSRTTrB(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int i,j,k;

	*comb = 0;
	for (i = 1; i <= n-1; i++) {
		for (j = i+1; j <= n; j++) {
			for(k = j+1; k <= n+1; k++) {
				vector[*comb] = nextTransposition(permutation,temp,copy,i,j,k);
				(*comb)++;
				vector[*comb] = nextSignedTransreversalB(permutation,temp,copy,i,j,k);
				(*comb)++;
			}
		}
	}

	for (i = 1; i <= n; i++) {
		for (j = i; j <= n; j++) {
			vector[*comb] = nextSignedReversal(permutation,temp,copy,i,j);
			(*comb)++;
		}
	}
}

void *loadVectorSTTrAB(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int i,j,k;

	*comb = 0;
	for (i = 1; i <= n-1; i++) {
		for (j = i+1; j <= n; j++) {
			for(k = j+1; k <= n+1; k++) {
				vector[*comb] = nextTransposition(permutation,temp,copy,i,j,k);
				(*comb)++;
				vector[*comb] = nextSignedTransreversalA(permutation,temp,copy,i,j,k);
				(*comb)++;
				vector[*comb] = nextSignedTransreversalB(permutation,temp,copy,i,j,k);
				(*comb)++;
			}
		}
	}
}

void *loadVectorCR(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int i,j;

	*comb = 0;
	for (i = 0; i <= n/2; i++) {
		vector[*comb] = nextCenteredReversal(permutation,temp,copy,i,i);
		(*comb)++;
	}

	for (i = 1; i <= assimetryLevel; i++) {
		for (j = 0; 2*j+i < n; j++) {
			vector[*comb] = nextCenteredReversal(permutation,temp,copy,j+i,j);
			(*comb)++;
			vector[*comb] = nextCenteredReversal(permutation,temp,copy,j,j+i);
			(*comb)++;
		}
	}
}

void *loadVectorSCR(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int i,j;

	*comb = 0;
	for (i = 0; i <= n/2; i++) {
		vector[*comb] = nextSignedCenteredReversal(permutation,temp,copy,i,i);
		(*comb)++;
	}

	for (i = 1; i <= assimetryLevel; i++) {
		for (j = 0; 2*j+i < n; j++) {
			vector[*comb] = nextSignedCenteredReversal(permutation,temp,copy,j+i,j);
			(*comb)++;
			vector[*comb] = nextSignedCenteredReversal(permutation,temp,copy,j,j+i);
			(*comb)++;
		}
	}
}

void *loadVectorSCRUSR(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	loadVectorSCR(n, comb, temp, copy, permutation, vector, assimetryLevel);

	int i;

	for (i = 1; i <= n; i++) {		
		vector[*comb] = nextSignedReversal(permutation,temp,copy,i,i);
		(*comb)++;
	}
}

void *loadVectorShSw(int n, int *comb, int *temp, int *copy, tPermutation *permutation, unsigned int *vector, int assimetryLevel){
	int i,j;

	*comb = 0;
	for(i = 1; i < n; i++) {
		for(j = i+1; j <= i+2 && j <= n; j++) {
			vector[*comb] = nextSwap(permutation,temp,copy,i,j);
			(*comb)++;		
		}		
	}
}
