/**********************************************************************************
 * Source file for the implementation of the algorithm for
 * computing rearrangement distance of all permutations of size n.
 * Author : Gustavo Rodrigues Galvao - May 2010.
**********************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "permutation.h"
#include "ranking.h"

tPermutation* newPermutation(int *elements, int n, int sign) {
	tPermutation *p;

	p = (tPermutation*) malloc(sizeof(tPermutation));
	p->n = n;
	p->elements = elements;
	p->sign = sign;

	return p;
}

void freePermutation(tPermutation *permutation) {
	free(permutation->elements);
	free(permutation);
}

void getIdentityPermutation(tPermutation *permutation) {
	int i;

	for (i = 0; i < permutation->n; i++) {
		permutation->elements[i] = i + 1;
	}
}

void getCopy(tPermutation *permutation, int *copy){
	int i;

	for(i = 0; i < permutation->n; i++){
		copy[i] = permutation->elements[i];
	}
}

void getUnsignedCopy(tPermutation *permutation, int *copy){
	int i;

	for(i = 0; i < permutation->n; i++){
		copy[i] = abs(permutation->elements[i]);
	}
}

void getInversePermutation(int *pi, int *invpi, int n){
	int i;

	for(i = 0; i < n; i++){
		invpi[pi[i]-1] = i+1;
	}
}

void applyTransposition(tPermutation *permutation, int *temp, int j, int k,
		int s) {
	int i;

	j--;
	k--;
	s--;

	for (i = 0; i < k - j; i++) {
		temp[i] = permutation->elements[j + i];
	}

	for (i = 0; i < s - k; i++) {
		permutation->elements[j + i] = permutation->elements[k + i];
	}

	for (i = 0; i < k - j; i++) {
		permutation->elements[j + s - k + i] = temp[i];
	}
}

void applyTransreversalA(tPermutation *permutation, int *temp, int j, int k,
		int s) {
	int i;

	j--;
	k--;
	s--;

	for (i = 0; i < k - j; i++) {
		temp[k - j - i - 1] = permutation->elements[j + i];
	}

	for (i = 0; i < s - k; i++) {
		permutation->elements[j + i] = permutation->elements[k + i];
	}

	for (i = 0; i < k - j; i++) {
		permutation->elements[j + s - k + i] = temp[i];
	}
}

void applySignedTransreversalA(tPermutation *permutation, int *temp, int j, int k,
		int s) {
	int i;

	j--;
	k--;
	s--;

	for (i = 0; i < k - j; i++) {
		temp[k - j - i - 1] = (-1)*permutation->elements[j + i];
	}

	for (i = 0; i < s - k; i++) {
		permutation->elements[j + i] = permutation->elements[k + i];
	}

	for (i = 0; i < k - j; i++) {
		permutation->elements[j + s - k + i] = temp[i];
	}
}


void applyTransreversalB(tPermutation *permutation, int *temp, int j, int k,
		int s) {
	int i;

	j--;
	k--;
	s--;

	for (i = 0; i < k - j; i++) {
		temp[i] = permutation->elements[j + i];
	}

	for (i = 0; i < s - k; i++) {
		temp[i + k - j] = permutation->elements[s - 1 - i];
	}

	for (i = 0; i < s - k; i++) {
		permutation->elements[j + i] = temp[i + k - j];
	}	

	for (i = 0; i < k - j; i++) {
		permutation->elements[j + s - k + i] = temp[i];
	}
}

void applySignedTransreversalB(tPermutation *permutation, int *temp, int j, int k,
		int s) {
	int i;

	j--;
	k--;
	s--;

	for (i = 0; i < k - j; i++) {
		temp[i] = permutation->elements[j + i];
	}

	for (i = 0; i < s - k; i++) {
		temp[i + k - j] = (-1)*permutation->elements[s - 1 - i];
	}

	for (i = 0; i < s - k; i++) {
		permutation->elements[j + i] = temp[i + k - j];
	}	

	for (i = 0; i < k - j; i++) {
		permutation->elements[j + s - k + i] = temp[i];
	}
}

void applyBlockInterchange(tPermutation *permutation, int *temp, int j, int k, int r, int s) {
	int i, dif;

	j--;
	k--;
	s--;
	r--;

	for (i = 0; i < k - j; i++) {
		temp[i] = permutation->elements[j + i];
	}

	for(i = 0; i < s - r;  i++){
		temp[k - j + i] = permutation->elements[r + i];	
	}

	for(i = 0; i < r - k;  i++){
		temp[k - j + s - r + i] = permutation->elements[k + i];	
	}

	if((k - j) > (s - r)){
		dif = (k - j) - (s - r);
		for(i = k; i < r; i++){
			permutation->elements[i - dif] = temp[k - j + s - r + i - k];		
		}	
	} else if((s - r) > (k - j)){
		dif = (s - r) - (k - j);
		for(i = k; i < r; i++){
			permutation->elements[i + dif] = temp[k - j + s - r + i - k];		
		}	
	}

	for (i = 0; i < s - r; i++) {
		permutation->elements[j + i] = temp[k - j + i];
	}

	for (i = 0; i < k - j; i++) {
		permutation->elements[j + s - k + i] = temp[i];
	}
}

void applyReversal(tPermutation *permutation, int *temp, int j, int k) {
	int i;

	j--;
	k--;

	for (i = k; i >= j; i--) {
		temp[k - i] = permutation->elements[i];
	}

	for (i = j; i <= k; i++) {
		permutation->elements[i] = temp[i - j];
	}
}

void applySignedReversal(tPermutation *permutation, int *temp, int j, int k) {
	int i;

	j--;
	k--;

	for (i = k; i >= j; i--) {
		temp[k - i] = (-1) * permutation->elements[i];
	}

	for (i = j; i <= k; i++) {
		permutation->elements[i] = temp[i - j];
	}
}

void applyCenteredReversal(tPermutation *permutation, int *temp, int j, int k) {
	int n;

	n = permutation->n;
	applyReversal(permutation,temp,j+1,n-k);
}

void applySignedCenteredReversal(tPermutation *permutation, int *temp, int j, int k) {
	int n;

	n = permutation->n;
	applySignedReversal(permutation,temp,j+1,n-k);
}

void applySwap(tPermutation *permutation, int *temp, int i, int j) {
	i--;
	j--;

	temp[0] = permutation->elements[i];
	permutation->elements[i] = permutation->elements[j];
	permutation->elements[j] = temp[0];
}

unsigned int rankUnsignedPermutation(tPermutation *permutation, int *inverse, int *copy){
	getCopy(permutation, copy);
	getInversePermutation(permutation->elements, inverse, permutation->n);
	return rank(permutation->n, copy, inverse);
}

void unrankUnsignedPermutation(tPermutation *permutation, unsigned int rank){
	getIdentityPermutation(permutation);
	unrank(permutation->n, rank, permutation->elements);
}

unsigned int rankSignedPermutation(tPermutation *permutation, int *inverse, int *copy){
	int i,n;
	unsigned int value;

	n = permutation->n;
	getUnsignedCopy(permutation, copy);
	getInversePermutation(copy, inverse, n);
	value = rank(permutation->n, copy, inverse)*base2[n];

	for(i = 0; i < n; i++){
		if(permutation->elements[i] < 0){
			value += base2[n - 1 - i];
		}
	}

	return value;
}

void unrankSignedPermutation(tPermutation *permutation, unsigned int rank){
	int i, n;
	unsigned int r;

	n = permutation->n;
	r = rank%base2[n];
	rank -= r;

	rank = rank/base2[n];

	getIdentityPermutation(permutation);
	unrank(permutation->n, rank, permutation->elements);

	for(i = n - 1; i >= 0; i--){
		if(r%2 == 1){
			permutation->elements[i] = (-1)*permutation->elements[i];
		}
		r = r/2;
	}
}

unsigned int rankPermutation(tPermutation *permutation, int *inverse, int *copy){
	if(permutation->sign == 0){
		return rankUnsignedPermutation(permutation,inverse,copy);
	} else {
		return rankSignedPermutation(permutation,inverse,copy);
	}
}

void unrankPermutation(tPermutation *permutation, unsigned int rank){
	if(permutation->sign == 0){
		unrankUnsignedPermutation(permutation,rank);
	} else {
		unrankSignedPermutation(permutation,rank);
	}
}
