/**********************************************************************************
 * Source file for the implementation of the algorithm for
 * computing rearrangement distance of all permutations of size n.
 * Author : Gustavo Rodrigues Galvao - May 2010.
**********************************************************************************/

#include "allPermutations.h"

void usage(){
	printf("Usage: allPermutations output_format permutation_size operation num_threads [assimetry_level]\n");
	printf("permutation_size: permutations number of elements.\n");
	printf("output_format: the two options are \"ascii\", for human readble output, and \"bin\", for binary output.\n");
	printf("operation: type of rearrangement event.\n");
	printf("num_threads: number of threads.\n");
	printf("assimetry_level: level of assymetry for almost-symmetric reversals. Default is one.\n");
	printf("Operation options\n");
	printf("-t: transposition\n");
	printf("-pt: prefix transposition\n");
	printf("-r: reversal\n");
	printf("-pr: prefix reversal\n");
	printf("-prpt: prefix reversal and prefix transposition\n");
	printf("-sr: signed reversal\n");
	printf("-psr: prefix signed reversal\n");
	printf("-rt: reversal and transposition\n");
	printf("-srt: signed reversal and transposition\n");
	printf("-bi: block-interchange\n");
	printf("-rttr: reversal, transposition and transreversal\n");
	printf("-srttrab: signed reversal, transposition and signed transreversal (a and b types)\n");
	printf("-srttra: signed reversal, transposition and signed transreversal (a type)\n");
	printf("-srttrb: signed reversal, transposition and signed transreversal (b type)\n");
	printf("-sttrab: transposition and signed transreversal (a and b types)\n");
	printf("-syr: symmetric reversal\n");
	printf("-ssyr: signed symmetric reversal\n");
	printf("-asyr: almost-symmetric reversal\n");
	printf("-sasyr: signed almost-symmetric reversal\n");
	printf("-sasyrur: signed almost-symmetric reversal and unitary signed reversals\n");
	printf("-shsw: short swap\n");
}

int main(int argc, char **argv){
	int n, num_threads, assimetryLevel, printASCII;

	if(argc != 5 && argc != 6){
		usage();
		exit(1);
	} else if(!isnumeric(argv[2])) {
		usage();
		exit(1);
	} else if(!isnumeric(argv[4])){
		usage();
		exit(1);
	} else if(argv[5] != NULL && !isnumeric(argv[5])){
		usage();
		exit(1);
	} else {
		if(strcmp(argv[1],"ascii") == 0){
			printASCII = 1;
		} else if(strcmp(argv[1],"bin") == 0) {
			printASCII = 0;
		} else {
			usage();
			exit(1);
		}

		n = atoi(argv[2]);
		num_threads = atoi(argv[4]);
		
		if(argv[5] != NULL){
			assimetryLevel = atoi(argv[5]);
		} else {
			assimetryLevel = 1;
		}

		if(strcmp(argv[3],"-pt") == 0){
			generateAllPermutations(n,0,num_threads,assimetryLevel,printASCII,loadVectorPT);
		} else if(strcmp(argv[3],"-t") == 0){
			generateAllPermutations(n,0,num_threads,assimetryLevel,printASCII,loadVectorT);
		} else if(strcmp(argv[3],"-sr") == 0){
			generateAllPermutations(n,1,num_threads,assimetryLevel,printASCII,loadVectorSR);
		} else if(strcmp(argv[3],"-r") == 0){
			generateAllPermutations(n,0,num_threads,assimetryLevel,printASCII,loadVectorR);
		} else if(strcmp(argv[3],"-rt") == 0){
			generateAllPermutations(n,0,num_threads,assimetryLevel,printASCII,loadVectorRT);
		} else if(strcmp(argv[3],"-srt") == 0){
			generateAllPermutations(n,1,num_threads,assimetryLevel,printASCII,loadVectorSRT);
		} else if(strcmp(argv[3],"-pr") == 0){
			generateAllPermutations(n,0,num_threads,assimetryLevel,printASCII,loadVectorPR);
		} else if(strcmp(argv[3],"-prpt") == 0){
			generateAllPermutations(n,0,num_threads,assimetryLevel,printASCII,loadVectorPRPT);
		} else if(strcmp(argv[3],"-psr") == 0){
			generateAllPermutations(n,1,num_threads,assimetryLevel,printASCII,loadVectorPSR);
		} else if(strcmp(argv[3],"-bi") == 0){
			generateAllPermutations(n,0,num_threads,assimetryLevel,printASCII,loadVectorBI);
		} else if(strcmp(argv[3],"-rttr") == 0){
			generateAllPermutations(n,0,num_threads,assimetryLevel,printASCII,loadVectorRTTr);
		} else if(strcmp(argv[3],"-srttrab") == 0){
			generateAllPermutations(n,1,num_threads,assimetryLevel,printASCII,loadVectorSRTTrAB);
		} else if(strcmp(argv[3],"-srttra") == 0){
			generateAllPermutations(n,1,num_threads,assimetryLevel,printASCII,loadVectorSRTTrA);
		} else if(strcmp(argv[3],"-srttrb") == 0){
			generateAllPermutations(n,1,num_threads,assimetryLevel,printASCII,loadVectorSRTTrB);
		} else if(strcmp(argv[3],"-sttrab") == 0){
			generateAllPermutations(n,1,num_threads,assimetryLevel,printASCII,loadVectorSTTrAB);
		} else if(strcmp(argv[3],"-asyr") == 0){
			generateAllPermutations(n,0,num_threads,assimetryLevel,printASCII,loadVectorCR);
		} else if(strcmp(argv[3],"-sasyr") == 0){
			generateAllPermutations(n,1,num_threads,assimetryLevel,printASCII,loadVectorSCR);
		} else if(strcmp(argv[3],"-sasyrur") == 0){
			generateAllPermutations(n,1,num_threads,assimetryLevel,printASCII,loadVectorSCRUSR);
		} else if(strcmp(argv[3],"-syr") == 0){
			generateAllPermutations(n,0,num_threads,0,printASCII,loadVectorCR);
		} else if(strcmp(argv[3],"-ssyr") == 0){
			generateAllPermutations(n,1,num_threads,0,printASCII,loadVectorSCR);
		} else if(strcmp(argv[3],"-shsw") == 0){
			generateAllPermutations(n,0,num_threads,assimetryLevel,printASCII,loadVectorShSw);
		} else {
			usage();
			exit(1);
		}
	}

	return 0;
}
