/**********************************************************************************
 * Header file for the implementation of the algorithm for
 * computing rearrangement distance of all permutations of size n.
 * Author : Gustavo Rodrigues Galvao - May 2010.
**********************************************************************************/

#ifndef PERMUTATION
#define PERMUTATION

unsigned long *base2;

typedef struct tPermutation{
	int n, sign;
	int *elements;
}tPermutation;

tPermutation* newPermutation(int *elements, int n, int sign);

void freePermutation(tPermutation *permutation);

void applyTransposition(tPermutation *permutation, int *temp, int j, int k, int s);

void applyTransreversalA(tPermutation *permutation, int *temp, int j, int k, int s);

void applySignedTransreversalA(tPermutation *permutation, int *temp, int j, int k, int s);

void applyTransreversalB(tPermutation *permutation, int *temp, int j, int k, int s);

void applySignedTransreversalB(tPermutation *permutation, int *temp, int j, int k, int s);

void applyBlockInterchange(tPermutation *permutation, int *temp, int j, int k, int r, int s);

void applyReversal(tPermutation *permutation, int *temp, int j, int k);

void applySignedReversal(tPermutation *permutation, int *temp, int j, int k);

void applyCenteredReversal(tPermutation *permutation, int *temp, int j, int k);

void applySignedCenteredReversal(tPermutation *permutation, int *temp, int j, int k);

void applySwap(tPermutation *permutation, int *temp, int i, int j);

unsigned long rankPermutation(tPermutation *permutation, int *inverse, int *copy);

void unrankPermutation(tPermutation *permutation, unsigned long rank);

void getIdentityPermutation(tPermutation *permutation);

void getPermutationCopy(tPermutation *permutation, int *copy);

void getInversePermutation(int *pi, int *invpi, int n);
#endif
